#!/usr/bin/env python
import os
import subprocess
import sys
import tarfile
import shlex
import inspect

from hdputils import install_base_pkg, configureYarnClient, config_all_nodes, \
                    setHadoopEnvVar, home, hdpScript, configureJAVA, config_all_nodes
from bdutils import setDirPermission, fileSetKV, append_bashrc, is_jvm_service_active, setHadoopConfigXML, chownRecursive, kill_java_process

from shutil import rmtree, copyfile, copy
from socket import gethostname
from charmhelpers.core.hookenv import log, Hooks, relation_get, relation_set, unit_get, open_port
from charmhelpers.core.host import service_start, service_stop, add_user_to_group
from time import sleep


# Required for unit tests... :(
try:
    from charmhelpers.fetch import apt_install, apt_update,  apt_purge
except:
    pass

def createHDPHiveConf():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
   
    HIVE_CONF_DIR = os.environ["HIVE_CONF_DIR"]
    HDPConfPath = os.path.join(os.path.sep,home, hdpScript, "configuration_files", "hive")
    source = os.listdir(HDPConfPath)
    if os.path.islink(HIVE_CONF_DIR):
        os.unlink(HIVE_CONF_DIR)
    else:
        rmtree(HIVE_CONF_DIR)
    os.mkdir(HIVE_CONF_DIR)
    for files in source:
        srcFile = os.path.join(os.path.sep, HDPConfPath, files)
        desFile = os.path.join(os.path.sep, HIVE_CONF_DIR, files)
        copyfile(srcFile, desFile)
    subprocess.call(setPermHiveConfDir)

    
def uninstall_base_pkg():
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    packages = ['ntp',
                'openjdk-7-jdk',
                'hive',
                'hive-hcatalog',
                'mysql-connector-java*'
                ]
    apt_purge(packages)
    rmtree(hdpScriptPath)
    copyfile(bashrc+".org", bashrc)
 
   
def config_hive_nodes():
    import glob
    log("==> {}".format(inspect.stack()[0][3]),"INFO")
    config_all_nodes()
    setHadoopEnvVar()
    log("==> calling subprocess.call(hdp_hellper_script)","INFO")
    subprocess.call(hdp_hellper_script)
    os.symlink(glob.glob("/usr/share/java/mysql-connector-java-*.jar")[0],\
                         "/usr/lib/hive/lib/mys.jar")
    setDirPermission(os.environ['HIVE_LOG_DIR'], os.environ['HIVE_USER'], \
                     os.environ['HADOOP_GROUP'], 0755)
    copy( hadoopHDPConfPath, os.environ['HADOOP_CONF_DIR'])
    add_user_to_group("ubuntu", os.environ['HADOOP_GROUP'])
    add_user_to_group("ubuntu", "mapred")
    setDirPermission(os.path.join(os.path.sep,"user", "hive", "warehouse"), 'ubuntu',os.environ['HADOOP_GROUP'], 0777)
    setDirPermission(os.path.join(os.path.sep,"apps", "hive", "warehouse"), 'ubuntu','ubuntu', 0755)
    cmd = "chsh -s /bin/bash hive"
    subprocess.call(cmd.split())

    
def callHDFS_fs(command):
    hadoopPath = os.path.join(os.path.sep, 'usr', 'lib', 'hadoop', 'bin', 'hadoop')
    cmd = shlex.split("su hdfs -c '{} fs {}'".format(hadoopPath, command))
    subprocess.call(cmd)


def configurMySQLClient():
    host = relation_get('host')
    database = relation_get('database')
    user = relation_get('user')
    password = relation_get('password')
    log("HIVE ==> configurMySQLClient, host={h}, database={d}, user={u}, password={p}".format(h=host, u=user, p=password, d=database),"INFO")

    hiveConfPath = os.path.join(os.path.sep, os.environ['HIVE_CONF_DIR'],"hive-site.xml")
    setHadoopConfigXML(hiveConfPath,"javax.jdo.option.ConnectionURL" ,"jdbc:mysql://{h}/{d}?createDatabaseIfNotExist=true".format(h=host,d=database) )
    setHadoopConfigXML(hiveConfPath,"javax.jdo.option.ConnectionUserName" ,user )
    setHadoopConfigXML(hiveConfPath,"javax.jdo.option.ConnectionPassword" ,password )
    setHadoopConfigXML(hiveConfPath,"javax.jdo.option.ConnectionDriverName" ,"com.mysql.jdbc.Driver" )

    
def init_derby_Metastore_database_schema():
    cmd = os.path.join(os.path.sep, "usr", "lib","hive","bin","schematool")+ " -initSchema -dbType derby"
    subprocess.call(cmd.split())
    
def init_MYSQL_Metastore_database_schema():
    cmd = os.path.join(os.path.sep, "usr", "lib","hive","bin","schematool")+ " -initSchema -dbType mysql"
    subprocess.call(cmd.split())
    
def start_hive_metastore():
    cmd = shlex.split("su hive -c 'nohup hive --service metastore>{h}/hive.out 2>{h}/hive.log & '".format(h=os.environ['HIVE_LOG_DIR']))
    subprocess.call(cmd)
    
def start_hive_server2():
    log("==> start_hive_server ")
    hiverserverPath = os.path.join(os.path.sep, 'usr', 'lib', 'hive', 'bin', 'hiveserver2')
    cmd = shlex.split("su hive -c ' {h} > {l}/hiveserver2.out 2> {l}/hiveserver2.log &'".format(h=hiverserverPath, l=os.environ['HIVE_LOG_DIR']))
    log("==> start_hive_server cmd = {}".format(cmd))
    o = subprocess.check_output(cmd)
    log ("==>start_hive_server = result={}".format(o))
    
def restart_hive_server():
    kill_java_process("RunJar")
    # give time for JAVA GC
    sleep(5)
    start_hive_metastore()
    start_hive_server2()
    
    
#################################### Global Data ################################     


hdp_hellper_script = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files', 'scripts', "HDPHelperScripts.sh")
setPermHiveConfDir = os.path.join(os.path.sep, os.environ['CHARM_DIR'],'files', 'scripts', "setPermHiveConfDir.sh")
hadoopHDPConfPath = os.path.join(os.path.sep,home, hdpScript, "configuration_files", "core_hadoop","hadoop-env.sh")
hdpScript = "hdp_scripts"
hdpScriptPath = os.path.join(os.path.sep,home, hdpScript,'scripts')
bashrc = os.path.join(os.path.sep, home, '.bashrc')
# hadoopMemoryOptimizationData = os.path.join(os.path.sep, hdpScriptPath, "hdpMemOpt.txt");

##########################################################################################



# Start the Hook Logic Block
hooks = Hooks()


@hooks.hook('install')
def install():
    packages = ['ntp',
                'openjdk-7-jdk',
                'hive',
                'hive-hcatalog',
                'mysql-connector-java*']
    install_base_pkg(packages)
    config_hive_nodes()
    open_port(10000)
    init_derby_Metastore_database_schema()
    start_hive_metastore()
    start_hive_server2()
    

@hooks.hook('resourcemanager-relation-changed')
def resourcemanager_relation_changed():
    log ("HIVE ==> resourcemanager_relation_changed","INFO")
    resourceManagerReady  = relation_get("resourceManagerReady")
    if not resourceManagerReady:
        log ("HIVE ==> resourcemanager not ready","INFO")
        sys.exit(0)
    log("PIG ==> resourcemanager IP={}".format(relation_get('private-address')),"INFO")
    setHadoopEnvVar()
    configureYarnClient(relation_get('private-address'))
    restart_hive_server()
    
    
       
@hooks.hook('resourcemanager-relation-broken')
def resourcemanager_relation_broken():
    log ("Configuring resourcemanager - broken phase","INFO")
    setHadoopEnvVar()
      

@hooks.hook('namenode-relation-changed')
def namenode_relation_changed():
    log("HIVE==> Configuring namenode - Changed phase","INFO")

    nameNodeReady = relation_get("nameNodeReady")
    if not nameNodeReady:
        log("HIVE ==> namenode not ready- Changed phase","INFO")
        sys.exit(0)
    log("HIVE ==> namenode_IP={}".format(relation_get('private-address')),"INFO")
    setHadoopEnvVar()
    hdfsConfPath = os.path.join(os.path.sep, os.environ['HADOOP_CONF_DIR'],"core-site.xml")
    setHadoopConfigXML(hdfsConfPath, "fs.defaultFS", "hdfs://"+relation_get('private-address')+":8020")
    hive_user = os.environ['HIVE_USER']
    hdfs_user = os.environ['HDFS_USER']
    #Create the Hive user home directory on HDFS.
    callHDFS_fs("-mkdir -p /user/{}".format(hive_user))
    callHDFS_fs('-chown {h}:{d} /user/{h}'.format(h=hive_user, d=hdfs_user))
    # Create the warehouse directory on HDFS.
    callHDFS_fs('-mkdir -p /apps/hive/warehouse')
    callHDFS_fs('-chown -R {h}:{d} /apps/hive'.format(h=hive_user, d=hdfs_user))
    callHDFS_fs("-chmod -R 775 /apps/hive ")
    # Create the Hive scratch directory on HDFS.
    callHDFS_fs("-mkdir -p /tmp/scratch")
    callHDFS_fs("-chown -R {h}:{d} /tmp/scratch".format(h=hive_user, d=hdfs_user))
    callHDFS_fs("-chmod -R 777 /tmp/scratch ")
    restart_hive_server()
    
        
@hooks.hook('config-changed')
def config_changed():     
    log( "config-changed called", "INFO")  
    setHadoopEnvVar()

 

@hooks.hook('start')
def start():     
    log( "start called", "INFO")
      
@hooks.hook('stop')
def stop():     
    log( "stop called", "INFO")

@hooks.hook('db-relation-changed')
def db_relation_changed():
    log ("HIVE ==> db_relation_changed","INFO")
    database = relation_get('database')
    if not database:
        log("HIVE ==> db database relation is NOT ready", "INFO")
        sys.exit(0)
    #kill all hive processes
    kill_java_process("RunJar")
    setHadoopEnvVar()
    createHDPHiveConf()
    configurMySQLClient()
    init_MYSQL_Metastore_database_schema()
    start_hive_metastore()
    start_hive_server2()  

if __name__ == "__main__":
    hooks.execute(sys.argv)
